@@ -27,19 +27,20 @@ setClass("ArrayViewport",
     x_ranges <- x@ranges
     x_refdim <- x@refdim
     if (length(x_ranges) != length(x_refdim))
-        return(wmsg2("'ranges' and 'refdim' slots must have the same length"))
+        return("'ranges' and 'refdim' slots must have the same length")
 
     ## Check that the viewport is contained in the reference array.
     x_start <- start(x_ranges)
     x_end <- end(x_ranges)
     if (!(all(x_start >= 1L) && all(x_end <= x_refdim)))
-        return(wmsg2("object represents a viewport that is not ",
-                     "within the bounds of the reference array"))
+        return(paste0("object represents a viewport that is not ",
+                      "within the bounds of the reference array"))
 
-    ## A viewport cannot be longer than 2^31-1.
+    ## A viewport cannot be longer than 2^31 - 1.
     x_dim <- width(x_ranges)
     if (prod(x_dim) > .Machine$integer.max)
-        return(wmsg2("a viewport cannot be longer than .Machine$integer.max"))
+        return("a viewport cannot be longer than .Machine$integer.max")
+
     TRUE
 }
 
@@ -291,15 +292,15 @@ get_RegularArrayGrid_dim <- function(refdim, spacings)
 {
     x_tickmarks <- x@tickmarks
     if (!is.list(x_tickmarks))
-        return(wmsg2("'tickmarks' slot must be a list"))
+        return("'tickmarks' slot must be a list")
     ok <- vapply(x_tickmarks, .valid_tickmarks, logical(1), USE.NAMES=FALSE)
     if (!all(ok))
-        return(wmsg2("each list element in 'tickmarks' slot must be a ",
-                     "sorted integer vector of non-negative values"))
+        return(paste0("each list element in 'tickmarks' slot must be a ",
+                      "sorted integer vector of non-negative values"))
     x_maxlen <- .get_ArbitraryArrayGrid_maxlength(x)
     if (x_maxlen > .Machine$integer.max)
-        return(wmsg2("grid is too coarse (all grid elements must have a ",
-                     "length <= .Machine$integer.max)"))
+        return(paste0("grid is too coarse (all grid elements must have a ",
+                      "length <= .Machine$integer.max)"))
     TRUE
 }
 setValidity2("ArbitraryArrayGrid", .validate_ArbitraryArrayGrid)
@@ -315,18 +316,17 @@ setValidity2("ArbitraryArrayGrid", .validate_ArbitraryArrayGrid)
     x_spacings <- x@spacings
     x_refdim <- x@refdim
     if (length(x_spacings) != length(x_refdim))
-        return(wmsg2("'spacings' and 'refdim' slots must have ",
-                     "the same length"))
+        return("'spacings' and 'refdim' slots must have the same length")
     if (!all(x_spacings <= x_refdim))
-        return(wmsg2("values in 'spacings' slot must be <= their ",
-                     "corresponding value in 'refdim' slot"))
+        return(paste0("values in 'spacings' slot must be <= their ",
+                      "corresponding value in 'refdim' slot"))
     if (any(x_spacings == 0L & x_refdim != 0L))
-        return(wmsg2("values in 'spacings' slot cannot be 0 unless their ",
-                     "corresponding value in 'refdim' slot is also 0"))
+        return(paste0("values in 'spacings' slot cannot be 0 unless their ",
+                      "corresponding value in 'refdim' slot is also 0"))
     x_maxlen <- .get_RegularArrayGrid_maxlength(x)
     if (x_maxlen > .Machine$integer.max)
-        return(wmsg2("grid is too coarse (all grid elements must have a ",
-                     "length <= .Machine$integer.max)"))
+        return(paste0("grid is too coarse (all grid elements must have a ",
+                      "length <= .Machine$integer.max)"))
     TRUE
 }
 setValidity2("RegularArrayGrid", .validate_RegularArrayGrid)
@@ -605,8 +605,11 @@ setMethod("downsample", "RegularArrayGrid",
     function(x, ratio=1L)
     {
         ratio <- .normarg_ratio(ratio, dim(x))
-        ans_spacings <- pmin(x@spacings * ratio, refdim(x))
-        RegularArrayGrid(refdim(x), ans_spacings)
+        x_refdim <- refdim(x)
+        ## We turn 'ratio' into a double vector to prevent a potential
+        ## integer overflow.
+        ans_spacings <- pmin(x@spacings * as.double(ratio), x_refdim)
+        RegularArrayGrid(x_refdim, ans_spacings)
     }
 )
 
