@@ -16,21 +16,22 @@
 ### 'x' is **trusted** to be a logical array-like object.
 ### Return an L-index (if 'arr.ind=FALSE') or M-index (if 'arr.ind=TRUE').
 ### Used in unit tests!
-BLOCK_which <- function(x, arr.ind=FALSE, grid=NULL)
+BLOCK_which <- function(x, arr.ind=FALSE, grid=NULL, as.sparse=NA)
 {
     if (!isTRUEorFALSE(arr.ind))
         stop("'arr.ind' must be TRUE or FALSE")
     FUN <- function(block, arr.ind) {
-        bid <- currentBlockId(block)
-        minor <- base::which(block)
+        bid <- currentBlockId()
+        ## Dispatch on which() method for array or SparseArraySeed.
+        minor <- which(block)
         major <- rep.int(bid, length(minor))
-        grid <- effectiveGrid(block)
+        grid <- effectiveGrid()
         Mindex <- mapToRef(major, minor, grid, linear=TRUE)
         if (arr.ind)
             return(Mindex)
         Mindex2Lindex(Mindex, refdim(grid))
     }
-    block_results <- blockApply(x, FUN, arr.ind, grid=grid)
+    block_results <- blockApply(x, FUN, arr.ind, grid=grid, as.sparse=as.sparse)
     if (arr.ind) {
         Mindex <- do.call(rbind, block_results)
         oo <- .Mindex_order(Mindex)
@@ -97,6 +98,9 @@ BLOCK_which <- function(x, arr.ind=FALSE, grid=NULL)
                 Mindex1 <- mapToRef(bid, minor, grid, linear=TRUE)
                 block_ans <- .extract_array_element(x, Mindex1)
             } else {
+                ## TODO: Use read_block( , as.sparse=NA) here once subsetting
+                ## a SparseArraySeed object by a linear index (L-index) is
+                ## supported.
                 block <- read_block(x, grid[[bid]])
                 block_ans <- block[minor]
             }
@@ -229,9 +233,14 @@ setMethod("drop", "DelayedArray",
     function(x)
     {
         perm <- which(dim(x) != 1L)
-        if (length(perm) <= 1L)
-            return(as.vector(x))  # in-memory realization
-        aperm(x, perm)
+        if (length(perm) >= 2L)
+            return(aperm(x, perm))
+        ## In-memory realization.
+        ## We want to propagate the names so we use 'as.array(x, drop=TRUE)'
+        ## rather than 'as.vector(x)' (both are equivalent on an Array
+        ## derivative with less than 2 effective dimensions except that
+        ## the former propagates the names and the latter doesn't).
+        as.array(x, drop=TRUE)
     }
 )
 
@@ -295,10 +304,10 @@ setMethod("[", "DelayedArray", .subset_DelayedArray)
 .BLOCK_dense2sparse <- function(x, grid=NULL)
 {
     FUN <- function(block, arr.ind) {
-        bid <- currentBlockId(block)
+        bid <- currentBlockId()
         minor <- base::which(block != 0L)
         major <- rep.int(bid, length(minor))
-        grid <- effectiveGrid(block)
+        grid <- effectiveGrid()
         nzindex <- mapToRef(major, minor, grid, linear=TRUE)
         nzdata <- block[minor]
         list(nzindex, nzdata)
@@ -315,9 +324,9 @@ setAs("DelayedArray", "SparseArraySeed",
     function(from) .BLOCK_dense2sparse(from)
 )
 
-.from_DelayedMatrix_to_dgCMatrix <- function(from)
+.from_DelayedMatrix_to_sparseMatrix <- function(from)
 {
-    ans <- as(.BLOCK_dense2sparse(from), "dgCMatrix")
+    ans <- as(.BLOCK_dense2sparse(from), "sparseMatrix")
     ## The above does NOT propagate the dimnames at the moment (the
     ## intermediate SparseArraySeed container cannot currently store them)
     ## so we propagate them explicitly.
@@ -326,8 +335,7 @@ setAs("DelayedArray", "SparseArraySeed",
         dimnames(ans) <- from_dimnames
     ans
 }
-setAs("DelayedMatrix", "dgCMatrix", .from_DelayedMatrix_to_dgCMatrix)
-setAs("DelayedMatrix", "sparseMatrix", .from_DelayedMatrix_to_dgCMatrix)
+setAs("DelayedMatrix", "sparseMatrix", .from_DelayedMatrix_to_sparseMatrix)
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
