@@ -8,17 +8,27 @@ setClass("SparseArraySeed",
     representation(
         dim="integer",     # This gives us dim() for free!
         nzindex="matrix",  # M-index of the nonzero data.
-        nzdata="vector"    # A vector of length 'nrow(nzindex)' containing
-                           # the nonzero data.
+        nzdata="vector",   # A vector (atomic or list) of length
+                           # 'nrow(nzindex)' containing the nonzero data.
+        dimnames="list"    # List with one list element per dimension. Each
+                           # list element must be NULL or a character vector.
+    ),
+    prototype(
+        dim=0L,
+        nzindex=matrix(integer(0), ncol=1L),
+        dimnames=list(NULL)
     )
 )
 
 ### API:
-### - Getters: dim(), length(), nzindex(), nzdata(), sparsity()
-### - dense2sparse(), sparse2dense()
-### - Based on sparse2dense(): extract_array(), as.array(), as.matrix()
-### - Based on dense2sparse(): coercion to SparseArraySeed
-### - Back and forth coercion between SparseArraySeed and dgCMatrix
+### - Getters/setters: dim(), length(), nzindex(), nzdata(),
+###                    dimnames(), dimnames<-()
+### - sparsity().
+### - dense2sparse(), sparse2dense().
+### - Based on sparse2dense(): extract_array(), as.array(), as.matrix().
+### - Based on dense2sparse(): coercion to SparseArraySeed.
+### - Back and forth coercion between SparseArraySeed and dgCMatrix or
+###   lgCMatrix objects from the Matrix package.
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -63,6 +73,9 @@ setClass("SparseArraySeed",
     if (!isTRUE(msg))
         return(msg)
     msg <- .validate_nzdata_slot(x)
+    if (!isTRUE(msg))
+        return(msg)
+    msg <- validate_dimnames_slot(x, x@dim)
     if (!isTRUE(msg))
         return(msg)
     TRUE
@@ -72,7 +85,7 @@ setValidity2("SparseArraySeed", .validate_SparseArraySeed)
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Getters
+### Getters/setters
 ###
 
 setGeneric("nzindex", function(x) standardGeneric("nzindex"))
@@ -81,7 +94,25 @@ setMethod("nzindex", "SparseArraySeed", function(x) x@nzindex)
 setGeneric("nzdata", function(x) standardGeneric("nzdata"))
 setMethod("nzdata", "SparseArraySeed", function(x) x@nzdata)
 
+setMethod("dimnames", "SparseArraySeed",
+    function(x) simplify_NULL_dimnames(x@dimnames)
+)
+
+setReplaceMethod("dimnames", "SparseArraySeed",
+    function(x, value)
+    {
+        x@dimnames <- normarg_dimnames(value, dim(x))
+        x
+    }
+)
+
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### sparsity()
+###
+
 setGeneric("sparsity", function(x) standardGeneric("sparsity"))
+
 setMethod("sparsity", "SparseArraySeed",
     function(x) { 1 - length(nzdata(x)) / length(x) }
 )
@@ -110,17 +141,21 @@ setMethod("sparsity", "SparseArraySeed",
     rep(nzdata, length.out=length.out)
 }
 
-SparseArraySeed <- function(dim, nzindex=NULL, nzdata=NULL, check=TRUE)
+SparseArraySeed <- function(dim, nzindex=NULL, nzdata=NULL, dimnames=NULL,
+                            check=TRUE)
 {
     if (!is.numeric(dim))
         stop(wmsg("'dim' must be an integer vector"))
     if (!is.integer(dim))
         dim <- as.integer(dim)
     if (is.null(nzindex)) {
-        if (!is.null(nzdata))
-            stop(wmsg("'nzdata' must be NULL when 'nzindex' is NULL"))
-        nzindex <- Lindex2Mindex(integer(0), dim)
-        nzdata <- integer(0)
+        if (is.null(nzdata)) {
+            nzdata <- logical(0)  # vector()
+        } else if (!(is.vector(nzdata) && length(nzdata) == 0L)) {
+            stop(wmsg("'nzdata' must be NULL or a vector of length 0 ",
+                      "when 'nzindex' is NULL"))
+        }
+        nzindex <- matrix(integer(0), ncol=length(dim))
     } else {
         if (!is.matrix(nzindex))
             stop(wmsg("'nzindex' must be a matrix"))
@@ -130,7 +165,9 @@ SparseArraySeed <- function(dim, nzindex=NULL, nzdata=NULL, check=TRUE)
             dimnames(nzindex) <- NULL
         nzdata <- .normarg_nzdata(nzdata, nrow(nzindex))
     }
+    dimnames <- normarg_dimnames(dimnames, dim)
     new2("SparseArraySeed", dim=dim, nzindex=nzindex, nzdata=nzdata,
+                            dimnames=dimnames,
                             check=check)
 }
 
@@ -139,16 +176,18 @@ SparseArraySeed <- function(dim, nzindex=NULL, nzdata=NULL, check=TRUE)
 ### dense2sparse() and sparse2dense()
 ###
 
-### 'x' must be an array-like object that supports subsetting by an M-index
-### subscript.
+### 'x' must be an array-like object that supports 'type()' and subsetting
+### by an M-index subscript.
 ### Return a SparseArraySeed object.
 dense2sparse <- function(x)
 {
     x_dim <- dim(x)
     if (is.null(x_dim))
         stop(wmsg("'x' must be an array-like object"))
-    nzindex <- which(x != 0L, arr.ind=TRUE)
-    SparseArraySeed(x_dim, nzindex, x[nzindex], check=FALSE)
+    ## Make sure to use 'type()' and not 'typeof()'.
+    zero <- vector(type(x), length=1L)
+    nzindex <- which(x != zero, arr.ind=TRUE)  # M-index
+    SparseArraySeed(x_dim, nzindex, x[nzindex], dimnames(x), check=FALSE)
 }
 
 ### 'sas' must be a SparseArraySeed object.
@@ -157,9 +196,11 @@ sparse2dense <- function(sas)
 {
     if (!is(sas, "SparseArraySeed"))
         stop(wmsg("'sas' must be a SparseArraySeed object"))
-    ans <- array(0L, dim=dim(sas))
-    ans[nzindex(sas)] <- nzdata(sas)
-    ans
+    sas_nzdata <- nzdata(sas)
+    zero <- vector(typeof(sas_nzdata), length=1L)
+    ans <- array(zero, dim=dim(sas))
+    ans[nzindex(sas)] <- sas_nzdata
+    set_dimnames(ans, dimnames(sas))
 }
 
 
@@ -167,13 +208,19 @@ sparse2dense <- function(sas)
 ### The is_sparse() and extract_sparse_array() generics
 ###
 
-### is_sparse() detects **structural** sparsity which is a qualitative
+### is_sparse() detects **structural** sparsity which is a global qualitative
 ### property of array-like object 'x' rather than a quantitative one.
 ### In other words it doesn't look at the data in 'x' to decide whether 'x'
 ### should be considered sparse or not. Said otherwise, it is NOT about
 ### quantitative sparsity measured by sparsity().
+### IMPORTANT: Seeds for which is_sparse() returns TRUE **must** support
+### extract_sparse_array().
 setGeneric("is_sparse", function(x) standardGeneric("is_sparse"))
 
+setGeneric("is_sparse<-", signature="x",
+    function(x, value) standardGeneric("is_sparse<-")
+)
+
 ### By default, nothing is considered sparse.
 setMethod("is_sparse", "ANY", function(x) FALSE)
 
@@ -284,23 +331,6 @@ setMethod("extract_array", "SparseArraySeed",
 )
 
 
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### is_sparse() and extract_sparse_array() methods for dgCMatrix objects
-###
-
-setMethod("is_sparse", "dgCMatrix", function(x) TRUE)
-
-.extract_sparse_array_from_dgCMatrix <- function(x, index)
-{
-    stopifnot(is(x, "dgCMatrix"))
-    x <- as(x, "SparseArraySeed")
-    .extract_sparse_array_from_SparseArraySeed(x, index)
-}
-setMethod("extract_sparse_array", "dgCMatrix",
-    .extract_sparse_array_from_dgCMatrix
-)
-
-
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Coercion to/from SparseArraySeed
 ###
@@ -309,6 +339,7 @@ setMethod("extract_sparse_array", "dgCMatrix",
 as.array.SparseArraySeed <- function(x, ...) sparse2dense(x)
 setMethod("as.array", "SparseArraySeed", as.array.SparseArraySeed)
 
+### S3/S4 combo for as.matrix.SparseArraySeed
 .from_SparseArraySeed_to_matrix <- function(x)
 {
     x_dim <- dim(x)
@@ -316,40 +347,67 @@ setMethod("as.array", "SparseArraySeed", as.array.SparseArraySeed)
         stop(wmsg("'x' must have exactly 2 dimensions"))
     sparse2dense(x)
 }
-
-### S3/S4 combo for as.matrix.SparseArraySeed
 as.matrix.SparseArraySeed <-
     function(x, ...) .from_SparseArraySeed_to_matrix(x, ...)
 setMethod("as.matrix", "SparseArraySeed", .from_SparseArraySeed_to_matrix)
 
-### Doesn't work on DelayedArray objects at the moment. See dense2sparse()
-### above.
 setAs("ANY", "SparseArraySeed", function(from) dense2sparse(from))
 
-### Going back and forth between SparseArraySeed and dgCMatrix:
+### Going back and forth between SparseArraySeed and dgCMatrix or lgCMatrix
+### objects from the Matrix package:
 
-.from_dgCMatrix_to_SparseArraySeed <- function(from)
-{
-    i <- from@i + 1L
-    j <- rep.int(seq_len(ncol(from)), diff(from@p))
-    nzindex <- cbind(i, j, deparse.level=0L)
-    SparseArraySeed(dim(from), nzindex, from@x, check=FALSE)
-}
-setAs("dgCMatrix", "SparseArraySeed", .from_dgCMatrix_to_SparseArraySeed)
-
-.from_SparseArraySeed_to_dgCMatrix <- function(from)
+.from_SparseArraySeed_to_sparseMatrix <- function(from)
 {
     from_dim <- dim(from)
     if (length(from_dim) != 2L)
         stop(wmsg("the ", class(from), " object to coerce to dgCMatrix ",
-                  "must have exactly 2 dimensions"))
+                  "or lgCMatrix must have exactly 2 dimensions"))
     i <- from@nzindex[ , 1L]
     j <- from@nzindex[ , 2L]
-    x <- from@nzdata
-    Matrix::sparseMatrix(i, j, x=x, dims=from_dim, dimnames=dimnames(from))
+    sparseMatrix2(from_dim, i, j, from@nzdata, dimnames=dimnames(from))
 }
-setAs("SparseArraySeed", "dgCMatrix", .from_SparseArraySeed_to_dgCMatrix)
-setAs("SparseArraySeed", "sparseMatrix", .from_SparseArraySeed_to_dgCMatrix)
+setAs("SparseArraySeed", "sparseMatrix", .from_SparseArraySeed_to_sparseMatrix)
+
+.make_SparseArraySeed_from_dgCMatrix_or_lgCMatrix <-
+    function(from, use.dimnames=TRUE)
+{
+    i <- from@i + 1L
+    j <- rep.int(seq_len(ncol(from)), diff(from@p))
+    ans_nzindex <- cbind(i, j, deparse.level=0L)
+    ans_dimnames <- if (use.dimnames) dimnames(from) else NULL
+    SparseArraySeed(dim(from), ans_nzindex, from@x, ans_dimnames, check=FALSE)
+}
+
+setAs("dgCMatrix", "SparseArraySeed",
+    function(from) .make_SparseArraySeed_from_dgCMatrix_or_lgCMatrix(from)
+)
+setAs("lgCMatrix", "SparseArraySeed",
+    function(from) .make_SparseArraySeed_from_dgCMatrix_or_lgCMatrix(from)
+)
+
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### is_sparse() and extract_sparse_array() methods for dgCMatrix and
+### lgCMatrix objects from the Matrix package
+###
+### TODO: Support more sparseMatrix derivatives (e.g. dgTMatrix, dgRMatrix)
+### as the need arises.
+###
+
+setMethod("is_sparse", "dgCMatrix", function(x) TRUE)
+setMethod("is_sparse", "lgCMatrix", function(x) TRUE)
+
+.extract_sparse_array_from_sparseMatrix <- function(x, index)
+{
+    sm <- subset_by_Nindex(x, index)  # a dgCMatrix or lgCMatrix object
+    .make_SparseArraySeed_from_dgCMatrix_or_lgCMatrix(sm, use.dimnames=FALSE)
+}
+setMethod("extract_sparse_array", "dgCMatrix",
+    .extract_sparse_array_from_sparseMatrix
+)
+setMethod("extract_sparse_array", "lgCMatrix",
+    .extract_sparse_array_from_sparseMatrix
+)
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -366,12 +424,14 @@ setAs("SparseArraySeed", "sparseMatrix", .from_SparseArraySeed_to_dgCMatrix)
     msg <- validate_perm(perm, a_dim)
     if (!isTRUE(msg))
         stop(wmsg(msg))
-    ans_dim <- a_dim[perm]
-    ans_dim[is.na(perm)] <- 1L
-    ans_nzindex <- a@nzindex[ , perm, drop=FALSE]
-    ans_nzindex[ , is.na(perm)] <- 1L
-    BiocGenerics:::replaceSlots(a, dim=ans_dim,
-                                   nzindex=ans_nzindex,
+    new_dim <- a_dim[perm]
+    new_dim[is.na(perm)] <- 1L
+    new_nzindex <- a@nzindex[ , perm, drop=FALSE]
+    new_nzindex[ , is.na(perm)] <- 1L
+    new_dimnames <- a@dimnames[perm]
+    BiocGenerics:::replaceSlots(a, dim=new_dim,
+                                   nzindex=new_nzindex,
+                                   dimnames=new_dimnames,
                                    check=FALSE)
 }
 
