@@ -18,6 +18,24 @@ seq2 <- function(to, by)
 }
 
 
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### bplapply2()
+###
+### A simple wrapper to BiocParallel::bplapply() that falls back to lapply()
+### if 'BPPARAM' is NULL.
+###
+
+bplapply2 <- function(X, FUN, ..., BPPARAM=NULL)
+{
+    if (is.null(BPPARAM))
+        return(lapply(X, FUN, ...))
+    if (!requireNamespace("BiocParallel", quietly=TRUE))
+        stop(wmsg("Couldn't load the BiocParallel package. Please ",
+                  "install the BiocParallel package and try again."))
+    BiocParallel::bplapply(X, FUN, ..., BPPARAM=BPPARAM)
+}
+
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Greatest common divisor and least common multiple of 2 integers
 ###
@@ -92,9 +110,13 @@ normarg_dimnames <- function(dimnames, dim)
         return(vector("list", length=ndim))
     if (!is.list(dimnames))
         stop(wmsg("the supplied 'dimnames' must be NULL or a list"))
-    if (length(dimnames) != ndim)
+    if (length(dimnames) < ndim) {
+        ## Append NULLs to the list.
+        length(dimnames) <- ndim
+    } else if (length(dimnames) > ndim) {
         stop(wmsg("the supplied 'dimnames' must have one list element ",
                   "per dimension"))
+    }
     lapply(setNames(seq_len(ndim), names(dimnames)),
         function(along) {
             dn <- dimnames[[along]]
@@ -469,23 +491,3 @@ user_option_is_set <- function(name)
     name %in% names(.Options)
 }
 
-
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### bplapply2()
-###
-### A simple wrapper to BiocParallel::bplapply() that propagates
-### base::.Options to the workers.
-### Should probably go in BiocParallel, or even better, replace bplapply().
-###
-
-bplapply2 <- function(X, FUN, ..., BPREDO=list(), BPPARAM=bpparam())
-{
-    FUN2 <- function(x, opts)
-    {
-        force(opts)
-        base::options(opts)
-        FUN(x, ...)
-    }
-    bplapply(X, FUN2, opts=base::.Options, BPREDO=BPREDO, BPPARAM=BPPARAM)
-}
-
